@@ -55,7 +55,7 @@ class Properties(object):
         return prop.text == 'true'
 
     def _string(self, prop):
-        if prop.attrib.get("notr", None) == "true":
+        if prop.get("notr", None) == "true":
             return self._cstring(prop)
         else:
             if prop.text is None:
@@ -64,12 +64,19 @@ class Properties(object):
             return QtGui.QApplication.translate(self.uiname, text,
                                                 None, QtGui.QApplication.UnicodeUTF8)
 
-
     def _cstring(self, prop):
         return str(prop.text)
 
     def _color(self, prop):
-        return QtGui.QColor(*int_list(prop))
+        args = int_list(prop)
+
+        # Handle the optional alpha component.
+        alpha = int(prop.get("alpha", "255"))
+
+        if alpha != 255:
+            args.append(alpha)
+
+        return QtGui.QColor(*args)
 
     def _rect(self, prop):
         return QtCore.QRect(*int_list(prop))
@@ -98,28 +105,53 @@ class Properties(object):
 
     def _palette(self, prop):
         palette = self.factory.createQObject("QPalette", "palette", (),
-                                                   is_attribute = False)
+                                                   is_attribute=False)
         for palette_elem in prop:
             sub_palette = getattr(QtGui.QPalette, palette_elem.tag.title())
-            for idx, color in enumerate(palette_elem):
-                palette.setColor(sub_palette, QtGui.QPalette.ColorRole(idx),
-                                 self._color(color))
+            for role, color in enumerate(palette_elem):
+                if color.tag == 'color':
+                    # Handle simple colour descriptions where the role is
+                    # implied by the colour's position.
+                    palette.setColor(sub_palette,
+                            QtGui.QPalette.ColorRole(role), self._color(color))
+                elif color.tag == 'colorrole':
+                    role = getattr(QtGui.QPalette, color.get('role'))
+                    brushstyle = getattr(QtCore.Qt, color[0].get('brushstyle'))
+                    color = color[0][0]
+
+                    brush = self.factory.createQObject("QBrush", "brush",
+                                                    (self._color(color), ),
+                                                    is_attribute=False)
+                    brush.setStyle(brushstyle)
+                    palette.setBrush(sub_palette, role, brush)
+                else:
+                    raise UnsupportedPropertyError, color.tag
+
         return palette
 
     #@needsWidget
     def _sizepolicy(self, prop, widget):
         values = [int(child.text) for child in prop]
-        sizePolicy = self.factory.createQObject(
-            "QSizePolicy", "sizePolicy",
-            (QtGui.QSizePolicy.Policy(values[0]), QtGui.QSizePolicy.Policy(values[1])),
-            is_attribute = False)
-        sizePolicy.setHorizontalStretch(values[2])
-        sizePolicy.setVerticalStretch(values[3])
+
+        if len(values) == 2:
+            # Qt v4.3.0 and later.
+            horstretch, verstretch = values
+            hsizetype = getattr(QtGui.QSizePolicy, prop.get('hsizetype'))
+            vsizetype = getattr(QtGui.QSizePolicy, prop.get('vsizetype'))
+        else:
+            hsizetype, vsizetype, horstretch, verstretch = values
+            hsizetype = QtGui.QSizePolicy.Policy(hsizetype)
+            vsizetype = QtGui.QSizePolicy.Policy(vsizetype)
+
+        sizePolicy = self.factory.createQObject("QSizePolicy", "sizePolicy",
+                (hsizetype, vsizetype), is_attribute=False)
+        sizePolicy.setHorizontalStretch(horstretch)
+        sizePolicy.setVerticalStretch(verstretch)
         sizePolicy.setHeightForWidth(widget.sizePolicy().hasHeightForWidth())
         return sizePolicy
     _sizepolicy = needsWidget(_sizepolicy)
 
-    # font needs special handling/conversion of all child elements
+    # font needs special handling/conversion of all child elements.
     _font_attributes = (("Family",    str),
                         ("PointSize", int),
                         ("Weight",    int),
@@ -128,10 +160,8 @@ class Properties(object):
                         ("StrikeOut", bool_),
                         ("Bold",      bool_))
 
-    #@needsWidget
-    def _font(self, prop, widget):
-        newfont = self.factory.createQObject("QFont", "font",
-                                                     (widget.font(), ),
+    def _font(self, prop):
+        newfont = self.factory.createQObject("QFont", "font", (),
                                                      is_attribute = False)
         for attr, converter in self._font_attributes:
             v = prop.findtext("./%s" % (attr.lower(),))
@@ -140,7 +170,9 @@ class Properties(object):
 
             getattr(newfont, "set%s" % (attr,))(converter(v))
         return newfont
-    _font = needsWidget(_font)
+
+    def _cursorShape(self, prop):
+        return getattr(QtCore.Qt, prop.text)
 
     def convert(self, prop, widget = None):
         try:
@@ -188,37 +220,50 @@ class Properties(object):
                 getattr(widget, "set%s%s" % (propname[0].upper(), propname[1:]))(
                     self.convert(prop, widget))
 
-    # delayed properties will be set after the whole widget tree has been populated
+    # SPECIAL PROPERTIES
+    # If a property has a well-known value type but needs special,
+    # context-dependent handling, the default behaviour can be overridden here.
+
+    # Delayed properties will be set after the whole widget tree has been
+    # populated.
     def _delay(self, widget, prop):
         propname = prop.attrib["name"]
         self.delayed_props.append(
             (getattr(widget, "set%s%s" % (propname[0].upper(), propname[1:])),
                                   self.convert(prop)))
 
-    # this properties will be set with a widget.setProperty call rather than
-    # calling the set<property> function
+    # These properties will be set with a widget.setProperty call rather than
+    # calling the set<property> function.
     def _setViaSetProperty(self, widget, prop):
         widget.setProperty(prop.attrib["name"], QtCore.QVariant(self.convert(prop)))
 
+    # Ignore the property.
+    def _ignore(self, widget, prop):
+        pass
 
-    # SPECIAL PROPERTIES
-    # If a property has a well-known value type but needs special, context-dependent
-    # handling, the default behaviour can be overridden here
+    # Define properties that use the canned handlers.
+    currentIndex = _delay
+    currentRow = _delay
 
-    currentIndex = currentRow = _delay
-    showDropIndicator = intValue = value = _setViaSetProperty # QLCDValue needs this
+    showDropIndicator = _setViaSetProperty
+    intValue = _setViaSetProperty
+    value = _setViaSetProperty
 
-    # buddy setting has to be done after the whole widget tree has been populated
-    # we can't use delay here because we cannot get the actual buddy yet
+    objectName = _ignore
+    leftMargin = _ignore
+    topMargin = _ignore
+    rightMargin = _ignore
+    bottomMargin = _ignore
+    horizontalSpacing = _ignore
+    verticalSpacing = _ignore
+
+    # buddy setting has to be done after the whole widget tree has been
+    # populated.  We can't use delay here because we cannot get the actual
+    # buddy yet.
     def buddy(self, widget, prop):
         self.buddies.append((widget, prop[0].text))
 
-    # not interested
-    # TODO: check if that works always!
-    def objectName(self, widget, prop):
-        pass
-    
-    # geometry has to be handled specially if set on the toplevel widget
+    # geometry has to be handled specially if set on the toplevel widget.
     def geometry(self, widget, prop):
         rect = self._rect(prop[0])
         if widget.objectName() == self.uiname:
@@ -227,19 +272,39 @@ class Properties(object):
             widget.setGeometry(rect)
 
     def orientation(self, widget, prop):
-        # if the class is a QFrame, it's a line
+        # If the class is a QFrame, it's a line.
         if widget.metaObject().className() == "QFrame":
             widget.setFrameShape(
                 {"Qt::Horizontal": QtGui.QFrame.HLine,
                  "Qt::Vertical"  : QtGui.QFrame.VLine}[prop[0].text])
-            # in Qt Designer, lines appear to be sunken, QFormBuilder loads
-            # them as such, uic generates plain lines. We stick to the look
-            # in Qt Designer
+
+            # In Qt Designer, lines appear to be sunken, QFormBuilder loads
+            # them as such, uic generates plain lines.  We stick to the look in
+            # Qt Designer.
             widget.setFrameShadow(QtGui.QFrame.Sunken)
         else:
             widget.setOrientation(self._enum(prop[0]))
 
-    # attribute isWrapping of QListView is named inconsistently,
-    # should be wrapping
+    # The isWrapping attribute of QListView is named inconsistently, it should
+    # be wrapping.
     def isWrapping(self, widget, prop):
         widget.setWrapping(self.convert(prop))
+
+    # This is a pseudo-property injected to deal with setContentsMargin()
+    # introduced in Qt v4.3.
+    def pyuicContentsMargins(self, widget, prop):
+        widget.setContentsMargins(*int_list(prop))
+
+    # This is a pseudo-property injected to deal with setHorizontalSpacing()
+    # and setVerticalSpacing() introduced in Qt v4.3.
+    def pyuicSpacing(self, widget, prop):
+        horiz, vert = int_list(prop)
+
+        if horiz == vert:
+            widget.setSpacing(horiz)
+        else:
+            if horiz >= 0:
+                widget.setHorizontalSpacing(horiz)
+
+            if vert >= 0:
+                widget.setVerticalSpacing(vert)
