@@ -1,18 +1,19 @@
 import sys
 import logging
+import os.path
 import re
 from itertools import count
 
 try:
-    from xml.etree.cElementTree import parse
+    from xml.etree.cElementTree import parse, SubElement
 except ImportError:
     try:
-        from cElementTree import parse
+        from cElementTree import parse, SubElement
     except ImportError:
         try:
-            from elementtree.ElementTree import parse
+            from elementtree.ElementTree import parse, SubElement
         except ImportError:
-            from PyQt4.elementtree.ElementTree import parse
+            from PyQt4.elementtree.ElementTree import parse, SubElement
         
 
 from exceptions import NoSuchWidgetError
@@ -54,14 +55,12 @@ class WidgetStack(list):
         if isinstance(item, QtGui.QWidget):
             self.topwidget = item
 
-
     def popLayout(self):
         layout = list.pop(self)
         DEBUG("pop layout %s %s" % (layout.metaObject().className(),
                                     layout.objectName()))
         return layout
 
-
     def popWidget(self):
         widget = list.pop(self)
         DEBUG("pop widget %s %s" % (widget.metaObject().className(),
@@ -75,10 +74,8 @@ class WidgetStack(list):
         DEBUG("new topwidget %s" % (self.topwidget,))
         return widget
 
-
     def peek(self):
         return self[-1]
-    
 
     def topIsLayout(self):
         return isinstance(self[-1], QtGui.QLayout)
@@ -111,7 +108,6 @@ class UIParser(object):
             self.name_suffixes[name] = count(1)
             return name
 
-
     def reset(self):
         try: self.wprops.reset()
         except AttributeError: pass
@@ -135,7 +131,6 @@ class UIParser(object):
         if is_attribute:
             setattr(self.toplevelWidget, name, obj)
         return obj
-
     
     def createWidget(self, elem):
         def widgetClass(elem):
@@ -211,7 +206,6 @@ class UIParser(object):
                 self.stack.topwidget.addDockWidget(
                     QtCore.Qt.DockWidgetArea(dwArea), widget)
 
-
     def createSpacer(self, elem):
         width = int(elem.findtext("property/size/width"))
         height = int(elem.findtext("property/size/height"))
@@ -228,8 +222,59 @@ class UIParser(object):
         if self.stack.topIsLayout():
             self.stack.peek().addItem(spacer, *elem.attrib["grid-position"])
 
-
     def createLayout(self, elem):
+        # Qt v4.3 introduced setContentsMargins() and separate values for each
+        # of the four margins which are specified as separate properties.  This
+        # doesn't really fit the way we parse the tree (why aren't the values
+        # passed as attributes of a single property?) so we create a new
+        # property and inject it.  However, if we find that they have all been
+        # specified and have the same value then we inject a different property
+        # that is compatible with older versions of Qt.
+        left = self.wprops.getProperty(elem, 'leftMargin', -1)
+        top = self.wprops.getProperty(elem, 'topMargin', -1)
+        right = self.wprops.getProperty(elem, 'rightMargin', -1)
+        bottom = self.wprops.getProperty(elem, 'bottomMargin', -1)
+
+        # Count the number of properties and if they had the same value.
+        def comp_property(m, so_far=-2, nr=0):
+            if m >= 0:
+                nr += 1
+
+                if so_far == -2:
+                    so_far = m
+                elif so_far != m:
+                    so_far = -1
+
+            return so_far, nr
+
+        margin, nr_margins = comp_property(left)
+        margin, nr_margins = comp_property(top, margin, nr_margins)
+        margin, nr_margins = comp_property(right, margin, nr_margins)
+        margin, nr_margins = comp_property(bottom, margin, nr_margins)
+
+        if nr_margins > 0:
+            if nr_margins == 4 and margin >= 0:
+                # We can inject the old margin property.
+                me = SubElement(elem, 'property', name='margin')
+                SubElement(me, 'number').text = str(margin)
+            else:
+                # We have to inject the new internal property.
+                cme = SubElement(elem, 'property', name='pyuicContentsMargins')
+                SubElement(cme, 'number').text = str(left)
+                SubElement(cme, 'number').text = str(top)
+                SubElement(cme, 'number').text = str(right)
+                SubElement(cme, 'number').text = str(bottom)
+
+        # We do the same for setHorizontalSpacing() and setVerticalSpacing().
+        horiz = self.wprops.getProperty(elem, 'horizontalSpacing', -1)
+        vert = self.wprops.getProperty(elem, 'verticalSpacing', -1)
+
+        if horiz >= 0 or vert >= 0:
+            # We inject the new internal property.
+            cme = SubElement(elem, 'property', name='pyuicSpacing')
+            SubElement(cme, 'number').text = str(horiz)
+            SubElement(cme, 'number').text = str(vert)
+
         classname = elem.attrib["class"]
         if self.stack.topIsLayout():
             parent = None
@@ -242,7 +287,6 @@ class UIParser(object):
         if self.stack.topIsLayout():
             self.stack.peek().addLayout(layout, *elem.attrib["grid-position"])
 
-
     def handleItem(self, elem):
         if self.stack.topIsLayout():
             elem[0].attrib["grid-position"] = gridPosition(elem)
@@ -288,12 +332,10 @@ class UIParser(object):
                 self.wprops.setProperties(item, elem)
                 w.setItem(int(elem.attrib["row"]), int(elem.attrib["column"]), item)
                              
-
                 
     def addAction(self, elem):
         self.actions.append((self.stack.topwidget, elem.attrib["name"]))
 
-
     def addHeader(self, elem):
         if isinstance(self.stack.topwidget, QtGui.QTreeWidget):
             self.stack.topwidget.headerItem().setText(
@@ -316,7 +358,6 @@ class UIParser(object):
             else:
                 self.stack.topwidget.setVerticalHeaderItem(self.row_counter, item)
                 self.row_counter += 1
-            
 
     def createAction(self, elem):
         self.setupObject("QAction", self.currentActionGroup or self.toplevelWidget,
@@ -327,7 +368,6 @@ class UIParser(object):
         self.currentActionGroup = action_group
         self.traverseWidgetTree(elem)
         self.currentActionGroup = None
-        
 
     widgetTreeItemHandlers = {
         "widget"    : createWidget,
@@ -340,6 +380,7 @@ class UIParser(object):
         "column"    : addHeader,
         "row"       : addHeader,
         }
+
     def traverseWidgetTree(self, elem):
         for child in iter(elem):
             try:
@@ -347,7 +388,6 @@ class UIParser(object):
             except KeyError, e: 
                 continue
             handler(self, child)
-            
 
     def createUserInterface(self, elem):
         # Get the names of the class and widget.
@@ -387,11 +427,9 @@ class UIParser(object):
                 elif not isinstance(action_obj, QtGui.QActionGroup):
                     widget.addAction(action_obj)
 
-
     def setDelayedProps(self):
         for func, args in self.wprops.delayed_props:
             func(args)
-
             
     def setBuddies(self):
         for widget, buddy in self.wprops.buddies:
@@ -402,7 +440,6 @@ class UIParser(object):
                 DEBUG("ERROR in ui spec: %s (buddy of %s) does not exist",
                       buddy, widget.objectName())
 
-
     def classname(self, elem):
         DEBUG("uiname is %s", elem.text)
         name = elem.text
@@ -414,7 +451,6 @@ class UIParser(object):
         self.wprops.uiname = name
         self.setContext(name)
 
-
     def setContext(self, context):
         """
         Reimplemented by a sub-class if it needs to know the translation
@@ -422,12 +458,10 @@ class UIParser(object):
         """
         pass
 
-
     def readDefaults(self, elem):
         self.defaults["margin"] = int(elem.attrib["margin"])
         self.defaults["spacing"] = int(elem.attrib["spacing"])
 
-
     def setTaborder(self, elem):
         try:
             lastwidget = getattr(self.toplevelWidget, elem[0].text)
@@ -438,7 +472,6 @@ class UIParser(object):
             self.toplevelWidget.setTabOrder(lastwidget, widget)
             lastwidget = widget
 
-
     def readResources(self, elem):
         """
         Read a "resources" tag and add the module to import to the parser's
@@ -450,9 +483,7 @@ class UIParser(object):
             # Assume our convention for naming the Python files generated by
             # pyrcc4.
             if loc and loc.endswith('.qrc'):
-                loc = loc[:-4] + '_rc'
-                self.resources.append(loc)
-
+                self.resources.append(os.path.basename(loc[:-4] + '_rc'))
 
     def createConnections(self, elem):
         def name2object(obj):
@@ -467,7 +498,6 @@ class UIParser(object):
                                                     conn.findtext("slot").split("(")[0]))
         QtCore.QMetaObject.connectSlotsByName(self.toplevelWidget)
 
-
     def customWidgets(self, elem):
         def header2module(header):
             """header2module(header) -> string
@@ -488,11 +518,9 @@ class UIParser(object):
             self.factory.addCustomWidget(classname,
                                      custom_widget.findtext("extends") or "QWidget",
                                      header2module(custom_widget.findtext("header")))
-                                     
 
     def createToplevelWidget(self, classname, widgetname):
         raise NotImplementedError, "must be overridden"
-
     
     # finalize will be called after the whole tree has been parsed and can be
     # overridden.
@@ -512,11 +540,12 @@ class UIParser(object):
             ("tabstops",      self.setTaborder),
             ("resources",     self.readResources),
         )
+
         document = parse(filename)
         version = document.getroot().attrib["version"]
         DEBUG("UI version is %s" % (version,))
-        # right now, only version 4.0 is supported, which is used
-        # by Qt 4.0 and 4.1 as of 1/2006
+        # Right now, only version 4.0 is supported, which is used up to at
+        # least Qt 4.3.
         assert version in ("4.0",)
         for tagname, actor in branchHandlers:
             elem = document.find(tagname)
